import copy
import re

from .base import BaseAPI, strip_html_tags
from .exceptions import DataException, SymbolNotFound


class EtfDb(BaseAPI):
    """
    EtfDb.com
    """

    def get_basic_info(self):
        """
        Returns basic info about the given ETF. The info is:

        - Issuer
        - Brand
        - Structure
        - Expense Ratio
        - Inception
        - Index Tracked
        - Category
        - Leveraged
        - Asset Class
        - Asset Class Size
        - Region (General)
        - Region (Specific)
        - Description
        - Assets Under Management

        :return: ETF basic info as a dict.
        :rtype: dict
        """

        # Get HTML.
        try:
            html = self._get(
                f"https://etfdb.com/etf/{self.symbol.upper()}/#etf-ticker-profile"
            )
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(
            r"class=\'ticker-assets\'.*?>(.*?)</div>[^<]*</div>", html.text, re.DOTALL
        )

        # Check if the HTML contains only two occurrences.
        if 0 == len(finds):
            raise SymbolNotFound
        if 2 < len(finds):
            raise DataException(
                "More that two occurrences found in HTML - don't know what to do now"
            )

        # Process 1st list.
        list_items = re.findall(r"<span.*?>(.*?)</span>", finds[0], re.DOTALL)
        list_items = [strip_html_tags(i) for i in list_items]
        data = dict(zip(list_items[::2], list_items[1::2]))

        # Process 2nd list.
        list_items = re.findall(r"<span.*?>(.*?)</span>", finds[1], re.DOTALL)
        list_items = [strip_html_tags(i) for i in list_items]
        data |= dict(zip(list_items[::2], list_items[1::2]))

        for key in copy.deepcopy(data).keys():
            if key not in [
                "Issuer",
                "Brand",
                "Structure",
                "Expense Ratio",
                "Inception",
                "Index Tracked",
                "Category",
                "Leveraged",
                "Asset Class",
                "Asset Class Size",
                "Region (General)",
                "Region (Specific)",
            ]:
                del data[key]

        # Fetch description.
        finds = re.findall(
            r"id='analyst-report'>.*?<p><p><p>(.+?)</p>", html.text, re.DOTALL
        )

        # Check if the HTML contains only ine occurrences.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one occurrences found in HTML - don't know what to do now"
            )

        data["Description"] = strip_html_tags(finds[0])

        # ASM
        finds = re.findall(
            r"AUM</span>[^<]+<span[^>]+>([^<]+)</span>", html.text, re.DOTALL
        )

        # Check if the HTML contains only one occurrences.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one occurrences found in HTML - don't know what to do now"
            )

        data["Assets Under Management"] = finds[0]

        # 52 week hi/low.
        finds = re.findall(
            r"52 Week Lo</span>[^<]+<span[^>]+>([^<]+)</span>", html.text, re.DOTALL
        )

        # Check if the HTML contains only one occurrences.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one occurrences found in HTML - don't know what to do now"
            )

        data["Year Low"] = finds[0]

        finds = re.findall(
            r"52 Week Hi</span>[^<]+<span[^>]+>([^<]+)</span>", html.text, re.DOTALL
        )

        # Check if the HTML contains only one occurrences.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one occurrences found in HTML - don't know what to do now"
            )

        data["Year High"] = finds[0]

        return data



import html
import json
import re

from .base import BaseAPI
from .exceptions import SymbolNotFound


class BarChart(BaseAPI):
    def get_ratings(self):
        """
        Returns ratings for past 4 months.
        Each months is a dict with status and it's values
        where values are in absolute number (number of analyst)
        and percents (ratio).

        Returns data in format:

        .. code-block:: json

            [
                {
                    "Strong Buy":{
                        "value":"14",
                        "percent":60.86956521739131
                    },
                    "Hold":{
                        "value":"9",
                        "percent":39.130434782608695
                    }
                },
                {
                    "Strong Buy":{
                        "value":"15",
                        "percent":65.21739130434783
                    },
                    "Hold":{
                        "value":"8",
                        "percent":34.78260869565217
                    }
                },
                {
                    "Strong Buy":{
                        "value":"15",
                        "percent":62.5
                    },
                    "Moderate Buy":{
                        "value":"1",
                        "percent":4.166666666666666
                    },
                    "Hold":{
                        "value":"8",
                        "percent":33.33333333333333
                    }
                },
                {
                    "Strong Buy":{
                        "value":"17",
                        "percent":73.91304347826086
                    },
                    "Moderate Buy":{
                        "value":"1",
                        "percent":4.3478260869565215
                    },
                    "Hold":{
                        "value":"5",
                        "percent":21.73913043478261
                    }
                }
            ]

        :raises SymbolNotFound: In case the page doesn't exist/returns error code.
        :return: List of each month data.
        :rtype: list
        """

        try:
            response = self._get(
                f"https://www.barchart.com/stocks/quotes/{self.symbol.upper()}/analyst-ratings"
            )
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(
            r'<analyst-rating-pie[^>]*data-content="([^"]+)"',
            response.text,
            re.DOTALL,
        )

        if finds:
            return [json.loads(html.unescape(find)) for find in finds]

        return []



import re

from .base import BaseAPI, HtmlTableParser
from .exceptions import DataException, SymbolNotFound


class StockAnalysis(BaseAPI):
    def get_etf_holdings(self):
        """
        Fetches ETF holdings table with following columns:

        - No.
        - Symbol
        - Name
        - % Weight
        - Shares
        """

        try:
            html = self._get(
                f"https://stockanalysis.com/etf/{self.symbol.lower()}/holdings/"
            )
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(r"<table.*?>.*?</table>", html.text, re.DOTALL)

        # Check if the HTML contains only one table.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one table found in HTML - don't know what to do now"
            )

        parser = HtmlTableParser(columns=5)
        # monkey patch - fixing the case where table header is
        # improperly parsed  - "%" has it's own column and the
        # last "shares" column is then not parsed.
        parser.feed(finds[0])
        parser.data[:6] = ("No.", "Symbol", "Name", "% Weight", "Shares")

        return parser.get_data()

    def get_current_price_change(self):
        """
        Fetches current market price inc. pre/post market
        prices/percent/value changes. Also returns current
        market state (pre-market, open, post-market, closed).

        Fetched stucture has following fields:

        - state (pre-market, open, post-market, closed)
        - pre_market
            - change
                - percents
                - value
            - value
        - current_market
            - change
                - percents
                - value
            - value
        - post_market
            - change
                - percents
                - value
            - value

        Returned dict looks like:

        .. code-block:: python

            {
                "state": "open",
                "pre_market": {
                    "change": {
                        "percents": -1.32476,
                        "value": -1.42001
                    },
                    "value": 105.77
                },
                "current_market": {
                    "change": {
                        "percents": -1.6046284000000002,
                        "value": -1.7200012
                    },
                    "value": 105.47
                },
                "post_market": {
                    "change": {
                        "percents": 0.0,
                        "value": 0.0
                    },
                    "value": 0.0
                }
            }

        :return: Current/Pre/Post market numbers (all are floats).
        :rtype: dict
        """

        def get_state(data):
            """
            Returns one of following market states:

            - open
            - closed
            - pre-market
            - post-market
            """

            # Is state "extended" (meaning closed)?
            if not data["e"]:
                return data["ms"].lower()
            else:
                # Converts "After-hours" to "post-market".
                if "after-hours" == data["es"].lower():
                    return "post-market"

                return data["es"].lower()

        try:
            response = self._get(
                f"https://stockanalysis.com/api/quotes/s/{self.symbol.lower()}"
            )
        except Exception as e:
            raise SymbolNotFound from e

        data = response.json()["data"]
        state = get_state(data)
        to_return = {
            "state": state,
        }

        # Pre-market data.
        if "pre-market" == state:
            to_return["pre_market"] = {
                "change": {
                    "percents": data["ecp"],
                    "value": data["ec"],
                },
                "value": data["ep"],
            }
        else:
            to_return["pre_market"] = {
                "change": {
                    "percents": 0,
                    "value": 0,
                },
                "value": 0,
            }

        # Market to_return.
        if "open" == data["ms"]:
            to_return["current_market"] = {
                "change": {
                    "percents": data["cp"],
                    "value": data["c"],
                },
                "value": data["p"],
            }

        else:
            to_return["current_market"] = {
                "change": {
                    "percents": 0,
                    "value": 0,
                },
                "value": 0,
            }

        # Post-market data.
        if "post-market" == state:
            to_return["post_market"] = {
                "change": {
                    "percents": data["ecp"],
                    "value": data["ec"],
                },
                "value": data["ep"],
            }

        else:
            to_return["post_market"] = {
                "change": {
                    "percents": 0,
                    "value": 0,
                },
                "value": 0,
            }

        return to_return
